﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using UchetLabBusinessLogic.BusinessLogic;
using UchetLabContracts.BindingModels;
using UchetLabContracts.BusinessLogicsContracts;
using UchetLabContracts.SearchModels;

namespace WinFormUchetLab
{
    public partial class FormChecker : Form
    {
        private readonly ICheckerLogic _checkerLogic;
        public FormChecker(ICheckerLogic checkerLogic)
        {
            _checkerLogic = new CheckerLogic(SingletonDatabase.CheckerStorage);
            InitializeComponent();
        }

        private void FormChecker_Load(object sender, EventArgs e)
        {
            LoadData();
        }
        private void LoadData()
        {
            try
            {
                var checkers = _checkerLogic.ReadList(new CheckerSearchModel());
                if (checkers == null)
                {
                    MessageBox.Show("не удалось загрузить данные", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dataGridView1.Rows.Clear();
                foreach (var difficulty in checkers)
                {
                    dataGridView1.Rows.Add();
                    dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells["Id"].Value = difficulty.Id;
                    dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells["FIO"].Value = difficulty.FIO;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Ошибка при загрузке данных: {ex.Message}", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void dataGridView1_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                // Получаем текущую строку
                DataGridViewRow currentRow = dataGridView1.Rows[e.RowIndex];

                // Проверяем, есть ли у строки ID (если его нет, считаем запись новой)
                if (currentRow.Cells["Id"].Value == null || Convert.ToInt32(currentRow.Cells["Id"].Value) == 0)
                {
                    // Создание новой записи
                    _checkerLogic.Create(new CheckerBindingModel()
                    {
                        FIO = currentRow.Cells["FIO"].Value.ToString() // Достаём текст из строки
                    });
                }
                else
                {
                    // Обновление существующей записи
                    _checkerLogic.Update(new CheckerBindingModel()
                    {
                        Id = Convert.ToInt32(currentRow.Cells["Id"].Value), // Достаём Id для обновления
                        FIO = currentRow.Cells["FIO"].Value.ToString() // Достаём текст из строки
                    });
                }
                // Сообщаем пользователю об успешном сохранении
                MessageBox.Show("Запись успешно сохранена", "Сохранение", MessageBoxButtons.OK, MessageBoxIcon.Information);

                // Отложенный вызов функции загрузки данных
                this.BeginInvoke(new MethodInvoker(LoadData));
            }
            catch (Exception ex)
            {
                // Выводим сообщение об ошибке
                MessageBox.Show($"Ошибка при сохранении записи: {ex.Message}", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void dataGridView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Insert)
            {
                // Добавляем новую строку
                dataGridView1.Rows.Add();
                dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells["FIO"]; // Ставим фокус на новую строку
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Delete && dataGridView1.SelectedRows.Count > 0)
            {
                // Подтверждение удаления
                DialogResult result = MessageBox.Show("Вы уверены, что хотите удалить выбранную запись?",
                                                      "Подтверждение удаления", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (result == DialogResult.Yes)
                {
                    try
                    {
                        foreach (DataGridViewRow row in dataGridView1.SelectedRows)
                        {
                            if (!row.IsNewRow) // Не даём удалять последнюю "пустую" строку
                            {
                                // Получаем Id из столбца "Id"
                                int id = Convert.ToInt32(row.Cells["Id"].Value); // Приводим значение Id к типу int

                                // Удаляем запись через логику
                                _checkerLogic.Delete(new CheckerBindingModel()
                                {
                                    Id = id
                                });

                                // Удаляем строку из DataGridView
                                dataGridView1.Rows.Remove(row);
                            }
                        }

                        // Сообщаем об успешном удалении
                        MessageBox.Show("Запись успешно удалена", "Удаление", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        // Выводим сообщение об ошибке
                        MessageBox.Show($"Ошибка при удалении записи: {ex.Message}", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                e.Handled = true;
            }
        }

        private void dataGridView1_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            string? userInput = e.FormattedValue.ToString();

            if (string.IsNullOrWhiteSpace(userInput))
            {
                MessageBox.Show("Нельзя сохранять пустую запись!", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true; // Отменяем завершение редактирования, если строка пустая
            }
        }
    }
}
