﻿using ControlsLibraryNet60.Selected;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using UchetLabContracts.BindingModels;
using UchetLabContracts.BusinessLogicsContracts;
using UchetLabContracts.SearchModels;

namespace WinFormUchetLab
{
    public partial class FormLab : Form
    {
        private readonly ILabLogic _labLogic;
        private readonly ICheckerLogic _checkerLogic;
        private int? _id;

        public int Id { set { _id = value; } }
        public FormLab(ILabLogic labLogic, ICheckerLogic checkerLogic)
        {
            _labLogic = labLogic;
            _checkerLogic = checkerLogic;
            InitializeComponent();
            customTextBox4Date1.pattern = "dd.MM.yyyy";
        }

        private void FormLab_Load(object sender, EventArgs e)
        {
            var listChecker = _checkerLogic.ReadList(null);
            if (listChecker != null)
            {
                foreach (var type in listChecker)
                {
                    controlSelectedListBoxSingle1.AddElement(type.FIO);
                }
            }

            if (!_id.HasValue)
            {
                return;
            }

            try
            {
                var lab = _labLogic.ReadElement(new LabSearchModel { Id = _id.Value });
                if (lab == null)
                {
                    return;
                }

                textBoxTask.Text = lab.LabTask;
                controlSelectedListBoxSingle1.SelectedElement = lab.Checker;
                textBoxImage.Text = lab.TaskImage;
                customTextBox4Date1.date = lab.CheckDate;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            customTextBox4Date1.pattern = "dd.MM.yyyy";

            if (string.IsNullOrEmpty(textBoxTask.Text))
            {
                MessageBox.Show("Заполните задание", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxImage.Text))
            {
                MessageBox.Show("Приложите картинку!", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(controlSelectedListBoxSingle1.SelectedElement))
            {
                MessageBox.Show("Выберите проверяющего!", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (customTextBox4Date1.date == null)
            {
                MessageBox.Show("Введите дату сдачи!", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                var model = new LabBindingModel
                {
                    Id = _id ?? 0,
                    LabTask = textBoxTask.Text,
                    TaskImage = textBoxImage.Text,
                    Checker = controlSelectedListBoxSingle1.SelectedElement,
                    CheckDate = customTextBox4Date1.date,
                };

                var operatingResult = _id.HasValue ? _labLogic.Update(model) : _labLogic.Create(model);
                if (!operatingResult)
                {
                    throw new Exception("Ошибка при создании сущности 'Счет'!");
                }

                MessageBox.Show("Создание сущности 'Лабораторная' прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void textBoxImage_Click(object sender, EventArgs e)
        {
            using (var dialog = new OpenFileDialog { Filter = "jpg|*.jpg" })
            {
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    textBoxImage.Text = dialog.FileName.ToString();
                }
            }
        }
    }
}
