using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using UchetLabBusinessLogic.BusinessLogic;
using UchetLabContracts.BusinessLogicsContracts;
using UchetLabContracts.StorageContracts;
using UchetLabDatabaseImplement.Implements;

namespace WinFormUchetLab
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;

        /// <summary>
        /// IoC-
        /// </summary>
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            // 
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                //option.AddNLog("nlog.config");
            });

            // IoC-, 
            services.AddTransient<ILabStorage, LabStorage>();
            services.AddTransient<ICheckerStorage, CheckerStorage>();

            // IoC-, -
            services.AddTransient<ILabLogic, LabLogic>();
            services.AddTransient<ICheckerLogic, CheckerLogic>();

            // IoC-,  
            services.AddTransient<FormMain>();
            services.AddTransient<FormLab>();
            services.AddTransient<FormChecker>();
        }
    }
}